# -*- coding: utf-8 -*-
"""Tests for the reporter_utils module, function 'init_get_cases_process'"""

from testrail_api_reporter.utils.reporter_utils import (  # pylint: disable=import-error,no-name-in-module
    init_get_cases_process,
)


def test_init_get_cases_process():
    """Test init_get_cases_process"""
    cases_list, first_run, criteria, response, retry = init_get_cases_process()

    assert isinstance(cases_list, list)
    assert cases_list == []
    assert first_run is True
    assert criteria is None
    assert response is None
    assert retry == 0



# -*- coding: utf-8 -*-
"""Tests for the case_stat module, сlass 'CaseStat'"""

import pytest
from faker import Faker

from testrail_api_reporter.utils.case_stat import CaseStat  # pylint: disable=import-error,no-name-in-module

fake = Faker()


def test_case_stat_init():
    """Check default CaseStat init"""
    name = fake.word()
    case_stat = CaseStat(name)
    assert case_stat.name == name
    assert case_stat.name == name
    assert case_stat.total == 0
    assert case_stat.automated == 0
    assert case_stat.not_automated == 0
    assert case_stat.not_applicable == 0


def test_case_stat_name():
    """Check name attribute"""
    name = fake.word()
    case_stat = CaseStat(name)

    assert case_stat.get_name() == name, "Test case name does not match"

    new_name = fake.word()
    case_stat.set_name(new_name)
    assert case_stat.get_name() == new_name, "Test case name does not match after change"


def test_case_stat_total(case_stat):
    """Check total"""
    total = fake.random_number()
    case_stat.set_total(total)
    assert case_stat.get_total() == total, "Total number of test cases does not match"


def test_case_stat_automated(case_stat):
    """Check automated"""
    automated = fake.random_number()
    case_stat.set_automated(automated)
    assert case_stat.get_automated() == automated, "Number of automated test cases does not match"


def test_case_stat_not_automated(case_stat):
    """Check not automated"""
    not_automated = fake.random_number()
    case_stat.set_not_automated(not_automated)
    assert case_stat.get_not_automated() == not_automated, "Number of not automated test cases does not match"


def test_case_stat_not_applicable(case_stat):
    """Check not applicable"""
    not_applicable = fake.random_number()
    case_stat.set_not_applicable(not_applicable)
    assert case_stat.get_not_applicable() == not_applicable, "Number of not applicable test cases does not match"


def test_case_stat_negative_total(case_stat):
    """Negative case for total"""
    with pytest.raises(ValueError):
        case_stat.set_total(-1)


def test_case_stat_total_not_a_number(case_stat):
    """Negative case for total - incorrect type"""
    with pytest.raises(TypeError):
        case_stat.set_total("not a number")  # type: ignore


def test_case_stat_negative_automated(case_stat):
    """Negative case for automated"""
    with pytest.raises(ValueError):
        case_stat.set_automated(-1)


def test_case_stat_automated_not_a_number(case_stat):
    """Negative case for automated - incorrect type"""
    with pytest.raises(TypeError):
        case_stat.set_automated("not a number")  # type: ignore


def test_case_stat_negative_not_automated(case_stat):
    """Negative case for not_automated"""
    with pytest.raises(ValueError):
        case_stat.set_not_automated(-1)


def test_case_stat_not_automated_not_a_number(case_stat):
    """Negative case for not automated - incorrect type"""
    with pytest.raises(TypeError):
        case_stat.set_not_automated("not a number")  # type: ignore


def test_case_stat_negative_not_applicable(case_stat):
    """Negative case for not applicable"""
    with pytest.raises(ValueError):
        case_stat.set_not_applicable(-1)


def test_case_stat_not_applicable_not_a_number(case_stat):
    """Negative case for not applicable - incorrect type"""
    with pytest.raises(TypeError):
        case_stat.set_not_applicable("not a number")



# -*- coding: utf-8 -*-
""" This module contains service functions for reporter """
from logging import Logger
from os import popen
from typing import Optional, Any, Union

import requests


def format_error(error: Union[list, str, Exception]) -> str:
    """
    Service function for parse errors to human-readable format

    :param error: initial error
    :return: formatted string with error details
    """
    err_msg = ""
    error = error if isinstance(error, list) else [error]
    for err in error:
        err_msg = f"{err_msg} : {err}"
    return err_msg


def upload_image(filename: str, api_token: str) -> dict:
    """
    Service function to upload images to third-party image hosting

    :param filename: filename or path to image, which should be uploaded
    :param api_token: unique API token for image upload on https://freeimage.host
    :return: dict with urls with image itself and its thumbnail
    """
    payload = {"action": "upload", "key": api_token, "format": "json"}
    with open(filename, "rb") as source_file:
        response = requests.post(
            url="https://freeimage.host/api/1/upload",
            data=payload,
            timeout=5,
            verify=True,
            files={"source": source_file},
        )
    return {
        "image": response.json()["image"]["url"],
        "thumb": response.json()["image"]["thumb"]["url"],
    }


def delete_file(filename: str, debug: bool = True, logger: Optional[Logger] = None):
    """
    Service function to delete file from filesystem

    :param filename: filename or path to file, which should be deleted
    :param debug: debug output is enabled, may be True or False, optional, by default, is True
    :param logger: logger, optional
    """
    popen(f"rm {filename}").read()
    if debug:
        if logger:
            logger.debug(f"Removed {filename}")


def zip_file(filename: str, suffix: Optional[str] = None, debug: bool = True, logger: Optional[Logger] = None) -> str:
    """
    Service function to ZIP file

    :param filename: filename or path to file, which should be zipped
    :param suffix: suffix for zipped file, optional
    :param debug: debug output is enabled may be True or False, optional, by default is True
    :param logger: logger, optional
    :return: zipped filename
    """
    if suffix is None:
        suffix = ""
    zipped_file = f'{filename.split(".")[0]}{suffix}.zip'
    popen(f"zip -r {zipped_file} {filename}").read()
    if debug:
        if logger:
            logger.debug(f"ZIPped {filename} to {zipped_file}")
    return zipped_file


def check_captions_and_files(
    captions: Union[list, None, Any], files: list, debug: bool = True, logger: Optional[Logger] = None
) -> Optional[list]:
    """
    Service function to check captions and file lists

    :param captions: list of captions for files, list of strings, if not provided, no captions will be added
    :param files: list of images urls
    :param debug: debug output is enabled, may be True or False, optional
    :param logger: logger object, optional
    :return: captions list or None
    """
    return_value = captions
    if not isinstance(captions, list) or not isinstance(files, list):
        if debug:
            if logger:
                logger.debug("Captions are not a list, thus no legend will be displayed")
        return_value = None
    elif len(captions) != len(files):
        if debug:
            if logger:
                logger.debug(
                    "Caption and file lists are not the same length %s != %s thus no legend will be displayed",
                    len(captions),
                    len(files),
                )
        return_value = None
    return return_value


def init_get_cases_process() -> tuple:
    """
    Service function to initialize a process

    :return: cases_list, first_run, criteria, response, retry
    """
    cases_list: list = []
    first_run = True
    criteria = None
    response = None
    retry = 0
    return cases_list, first_run, criteria, response, retry



# -*- coding: utf-8 -*-
"""Tests for the csv_parser module'"""
from datetime import datetime

import pytest
from faker import Faker

from testrail_api_reporter.utils.csv_parser import CSVParser  # pylint: disable=import-error,no-name-in-module

fake = Faker()


def test_save_history_data(csv_file, random_stat, case_stat):
    """Check save_history_data function"""
    parser = CSVParser(filename=csv_file)

    total, automated, not_automated, not_applicable = random_stat
    case_stat.set_total(total)
    case_stat.set_automated(automated)
    case_stat.set_not_automated(not_automated)
    case_stat.set_not_applicable(not_applicable)

    parser.save_history_data(report=case_stat)

    with open(csv_file, "r", encoding="utf-8") as readable_file:
        data = readable_file.read()
        assert data == (
            f"{datetime.today().strftime('%Y')},"
            f"{datetime.today().strftime('%m')},"
            f"{datetime.today().strftime('%d')},"
            f"{total},{automated},{not_automated},{not_applicable}\n"
        )


def test_save_history_data_no_filename():
    """No filename provided for save history data"""
    parser = CSVParser()

    with pytest.raises(ValueError):
        parser.save_history_data()


def test_save_history_data_no_report(csv_file):
    """No data for save history data"""
    parser = CSVParser(filename=csv_file)

    with pytest.raises(ValueError):
        parser.save_history_data()


def test_save_history_data_already_stored(csv_file, case_stat_random):
    """History already stored for such day for save history data"""
    parser = CSVParser(filename=csv_file)

    parser.save_history_data(report=case_stat_random)
    parser.save_history_data(report=case_stat_random)

    with open(csv_file, "r", encoding="utf-8") as readable_file:
        data = readable_file.read()
        assert data.count("\n") == 1



# -*- coding: utf-8 -*-
"""Tests for the reporter_utils module, function 'format_error'"""

from testrail_api_reporter.utils.reporter_utils import format_error  # pylint: disable=import-error,no-name-in-module


def test_format_error_single_string():
    """Test with single string error"""
    error = "Error occurred"
    expected_result = " : Error occurred"
    assert format_error(error) == expected_result


def test_format_error_list_with_single_error():
    """Test with single error in list"""
    error = ["Error occurred"]
    expected_result = " : Error occurred"
    assert format_error(error) == expected_result


def test_format_error_list_with_multiple_errors():
    """Test with multiple errors in list"""
    error = ["Error one", "Error two"]
    expected_result = " : Error one : Error two"
    assert format_error(error) == expected_result


def test_format_error_empty_list():
    """Test with empty list of errors"""
    error = []
    expected_result = ""
    assert format_error(error) == expected_result


def test_format_error_non_list_non_string():
    """Test with non-list and non-string error"""
    error = Exception("An exception occurred")
    expected_result = " : An exception occurred"
    assert format_error(error) == expected_result


def test_format_error_list_with_mixed_types():
    """Test with mixed types in list"""
    error = ["Error one", Exception("An exception occurred")]
    expected_result = " : Error one : An exception occurred"
    assert format_error(error) == expected_result


def test_format_error_list_with_empty_string():
    """Test with empty string in list"""
    error = ["Error one", ""]
    expected_result = " : Error one : "
    assert format_error(error) == expected_result


def test_format_error_list_with_empty_string_and_exception():
    """Test with empty string and exception in list"""
    error = ["Error one", "", Exception("An exception occurred")]
    expected_result = " : Error one :  : An exception occurred"
    assert format_error(error) == expected_result


def test_format_error_list_with_empty_string_and_exception_and_none():
    """Test with empty string, exception and None in list"""
    error = ["Error one", "", Exception("An exception occurred"), None]
    expected_result = " : Error one :  : An exception occurred : None"
    assert format_error(error) == expected_result


def test_format_error_list_with_none():
    """Test with None in list"""
    error = [None]
    expected_result = " : None"
    assert format_error(error) == expected_result


def test_format_error_none():
    """Test with None"""
    error = None
    expected_result = " : None"
    assert format_error(error) == expected_result


def test_format_error_value_error():
    """Test with ValueError"""
    error = ValueError("A value error occurred")
    expected_result = " : A value error occurred"
    assert format_error(error) == expected_result



# -*- coding: utf-8 -*-
""" testrail_api_reporter package """

# Engines
from .engines.at_coverage_reporter import ATCoverageReporter
from .engines.plotly_reporter import PlotlyReporter
from .engines.results_reporter import TestRailResultsReporter

# Publishers
from .publishers.confluence_sender import ConfluenceSender
from .publishers.email_sender import EmailSender
from .publishers.slack_sender import SlackSender
from .publishers.gdrive_uploader import GoogleDriveUploader

# Utils
from .utils.reporter_utils import upload_image, delete_file, zip_file
from .utils.logger_config import setup_logger



# -*- coding: utf-8 -*-
"""Tests for the reporter_utils module, function 'zip_file'"""

from os import path, remove
import logging
from unittest.mock import MagicMock
from testrail_api_reporter.utils.reporter_utils import zip_file  # pylint: disable=import-error,no-name-in-module


def test_zip_file_default(create_test_file):
    """Test zip file with default parameters"""
    zipped_file = ""
    try:
        zipped_file = zip_file(create_test_file, debug=False)
        assert path.exists(zipped_file) is True
    finally:
        if path.exists(zipped_file):
            remove(zipped_file)
        if path.exists(create_test_file):
            remove(create_test_file)


def test_zip_file_suffix(create_test_file):
    """Test zip file with custom suffix"""
    zipped_file = ""
    try:
        zipped_file = zip_file(create_test_file, suffix="_suffix", debug=False)
        assert path.exists(zipped_file) is True
    finally:
        if path.exists(zipped_file):
            remove(zipped_file)
        if path.exists(create_test_file):
            remove(create_test_file)


def test_zip_file_logger(create_test_file):
    """Test zip file with logger"""
    zipped_file = ""
    try:
        logger = logging.getLogger(__name__)
        logger.debug = MagicMock()
        zipped_file = zip_file(create_test_file, debug=True, logger=logger)
        assert path.exists(zipped_file) is True
        logger.debug.assert_called_once_with(f"ZIPped {create_test_file} to {zipped_file}")
    finally:
        if path.exists(zipped_file):
            remove(zipped_file)
        if path.exists(create_test_file):
            remove(create_test_file)



# -*- coding: utf-8 -*-
""" TestRails backup module """

import os
from datetime import datetime

from ..utils.logger_config import setup_logger, DEFAULT_LOGGING_LEVEL
from ..utils.reporter_utils import delete_file, zip_file


class TCBackup:
    """TestRails backup class"""

    def __init__(
        self,
        test_rails_url,
        test_rails_username,
        test_rails_password,
        test_rails_suite,
        cleanup_needed=True,
        backup_filename="backup.xml",
        cookie_name="cookie.txt",
        logger=None,
        log_level=DEFAULT_LOGGING_LEVEL,
    ):
        """
        General init

        :param test_rails_url: TestRails server url, string, i.e. https://wwakabobik.testrails.io
        :param test_rails_username: TestRails username, string, i.e. my@email.com, should have rights to make exports
        :param test_rails_password: TestRails password, string
        :param test_rails_suite: TestRails suite which needs to be downloaded, i.e. 42
        :param backup_filename: output backup file name, string, optional, by default it is backup.xml
        :param cookie_name: filename where TestRail cookie will be stored, string, by default is cookie.txt
        :param cleanup_needed: delete or not cookie file after backup, bool, True or False, by default is True
        :param logger: logger object, optional
        :param log_level: logging level, optional, by default is 'logging.DEBUG'
        """
        if not logger:
            self.___logger = setup_logger(name="TCBackup", log_file="TCBackup.log", level=log_level)
        else:
            self.___logger = logger
        self.___logger.debug("Initializing TestRails Backup")
        # TestRails
        self.__url = test_rails_url
        self.__username = test_rails_username
        self.__password = test_rails_password
        self.__suite = test_rails_suite
        # Service
        self.__cleanup_needed = cleanup_needed
        self.__backup_filename = backup_filename
        self.__cookie_name = cookie_name

    # TestRails part
    def __get_tr_cookie(self):
        """
        Login into TestRails and save session cookie

        :return: None
        """
        self.___logger.debug("Get cookie %s from %s for %s", self.__cookie_name, self.__url, self.__username)
        os.popen(
            f"curl -c {self.__cookie_name} "
            f'-H "Content-Type: application/x-www-form-urlencoded" '
            f'-d "name={self.__username}&password={self.__password}" -X POST '
            f'"{self.__url}/index.php?/auth/login"'
        ).read()

    def __download_tr_xml(self, filename=None, suite=None):
        """
        Download from TestRails XML file with testcases of testsuite

        :param filename: output backup file name, string, optional, by default it is backup.xml
        :param suite: TestRails suite which needs to be downloaded, i.e., 42
        :return: backup filename
        """
        if not filename:
            filename = self.__backup_filename
        if not suite:
            suite = self.__suite
        self.___logger.debug("Download XML %s from from %s", filename, self.__url)
        os.popen(
            f"curl -b {self.__cookie_name} " f'"{self.__url}/index.php?/suites/export/{suite}" ' f"--output {filename}"
        ).read()
        return filename

    def get_backup(self, filename=None, suite=None):
        """
        Download from TestRails backup file and deletes cookie if needed

        :param filename: output backup file name, string, optional, by default it is backup.xml
        :param suite: TestRails suite which needs to be downloaded, i.e., 42
        :return: backup filename
        """
        if not filename:
            filename = self.__backup_filename
        if not suite:
            suite = self.__suite
        self.__get_tr_cookie()
        backup_file = self.__download_tr_xml(filename=filename, suite=suite)
        if self.__cleanup_needed:
            delete_file(
                filename=self.__cookie_name,
                debug=self.___logger.debug == DEFAULT_LOGGING_LEVEL,
                logger=self.___logger,
            )
        return backup_file

    def get_archive_backup(self, filename=None, suite=None, suffix=f'_{datetime.today().strftime("%A")}'):
        """
        Download from TestRails backup file, add it to ZIP and deletes original backup file if needed

        :param filename: output backup file name, string, optional, by default it is backup.xml
        :param suite: TestRails suite which needs to be downloaded, i.e., 42
        :param suffix: suffix for backup archive, by default it "_DayOfWeek"
        :return: backup filename
        """
        if not filename:
            filename = self.__backup_filename
        if not suite:
            suite = self.__suite
        self.get_backup(filename=filename, suite=suite)
        backup_file = zip_file(
            filename=filename,
            suffix=suffix,
            debug=self.___logger.debug == DEFAULT_LOGGING_LEVEL,
            logger=self.___logger,
        )
        if self.__cleanup_needed:
            delete_file(filename=filename, debug=self.___logger.debug == DEFAULT_LOGGING_LEVEL)
        return backup_file



# -*- coding: utf-8 -*-
"""Tests for the reporter_utils module, function 'check_captions_and_files'"""

from logging import getLogger
from random import randint
from unittest.mock import MagicMock

from faker import Faker

from testrail_api_reporter.utils.reporter_utils import (  # pylint: disable=import-error,no-name-in-module
    check_captions_and_files,
)

faker = Faker()


def test_check_captions_and_files_not_list():
    """Test check_captions_and_files when captions is not a list"""
    captions = faker.sentence()
    files = [faker.file_name() for _ in range(randint(1, 10))]

    logger = getLogger(__name__)
    logger.debug = MagicMock()

    result = check_captions_and_files(captions, files, debug=True, logger=logger)
    assert result is None
    logger.debug.assert_called_once_with("Captions are not a list, thus no legend will be displayed")


def test_check_captions_and_files_different_length():
    """Test check_captions_and_files when captions and files have different lengths"""
    captions = [faker.sentence() for _ in range(randint(1, 10))]
    files = []
    while len(captions) == len(files):
        files = [faker.file_name() for _ in range(randint(1, 10))]

    logger = getLogger(__name__)
    logger.debug = MagicMock()

    result = check_captions_and_files(captions, files, debug=True, logger=logger)
    assert result is None
    logger.debug.assert_called_once_with(
        "Caption and file lists are not the same length %s != %s thus no legend will be displayed",
        len(captions),
        len(files),
    )


def test_check_captions_and_files_same_length():
    """Test check_captions_and_files when captions and files have the same length"""
    length = randint(1, 10)
    captions = [faker.sentence() for _ in range(length)]
    files = [faker.file_name() for _ in range(length)]

    result = check_captions_and_files(captions, files, debug=False)
    assert result == captions



# -*- coding: utf-8 -*-
"""Tests for plotly_reporter module, the PlotlyReporter clas, draw_test_case_by_area method"""

from copy import deepcopy
from os import path, remove, getcwd, environ
from random import choice, randint

import pytest
from faker import Faker

from testrail_api_reporter.engines.plotly_reporter import (  # pylint: disable=import-error,no-name-in-module
    PlotlyReporter,
)


fake = Faker()


@pytest.fixture
def random_expected_image(case_stat):
    """
    Fixture that chooses random expected image for draw automation state

    :param case_stat: fixture returns empty CaseStat object
    """
    case_stat_first = case_stat
    case_stat_first.set_name("UI")
    case_stat_first.total = 999
    case_stat_first.automated = 1234
    case_stat_first.not_automated = 99
    case_stat_first.not_applicable = 56
    case_stat_second = deepcopy(case_stat)
    case_stat_second.set_name("API")
    case_stat_second.total = 77
    case_stat_second.automated = 11
    case_stat_second.not_automated = 0
    case_stat_second.not_applicable = 1024
    return {
        "filename": f"{getcwd()}/tests/assets/expected_case_by_area.png",
        "data": [case_stat_first, case_stat_second],
    }


def test_draw_test_case_by_area_no_cases(random_plotly_reporter):
    """
    Init PlotlyReporter and call draw_test_case_by_area without cases should raise ValueError

    :param random_plotly_reporter: fixture returns PlotlyReporter
    """
    with pytest.raises(ValueError, match="No TestRail cases are provided, report aborted!"):
        random_plotly_reporter.draw_test_case_by_area(
            filename=fake.file_name(extension=choice(("png", "jpg", "jpeg", "webp")))
        )


def test_draw_test_case_by_area_no_filename(random_plotly_reporter):
    """
    Init PlotlyReporter and call draw_test_case_by_area without filename should raise ValueError

    :param random_plotly_reporter: fixture returns PlotlyReporter
    """
    with pytest.raises(ValueError, match="No output filename is provided, report aborted!"):
        random_plotly_reporter.draw_test_case_by_area(cases=[fake.pydict()])


def test_draw_test_case_by_area_creates_file(case_stat, case_stat_random, random_plotly_reporter):
    """
    Init PlotlyReporter and call draw_test_case_by_area with valid parameters should create file

    :param case_stat: fixture returns empty CaseStat object
    :param case_stat_random: fixture returns filled with random data CaseStat object
    :param random_plotly_reporter: fixture returns PlotlyReporter
    """
    filename = fake.file_name(extension=choice(("png", "jpg", "jpeg", "webp")))
    try:
        cases = [case_stat_random, case_stat]

        random_plotly_reporter.draw_test_case_by_area(filename=filename, cases=cases)

        assert path.exists(filename)
    finally:
        if path.exists(filename):
            remove(filename)


@pytest.mark.xfail(condition="GITHUB_ACTIONS" in environ, reason="Image may differ on GA env")
def test_draw_test_case_by_area_creates_correct_image(random_expected_image, compare_image):  # pylint: disable=W0621
    """
    Init PlotlyReporter and call draw_test_case_by_area with valid parameters should create correct image

    :param random_expected_image: fixture, returns any of possible expected cases
    :param compare_image: fixture, returns function to compare images
    """
    type_platforms = [{"name": "UI", "sections": [8080, 11, 4]}, {"name": "API", "sections": [12, 101, 86]}]
    filename = "actual_test_case_by_area.png"
    try:
        plotly_reporter = PlotlyReporter(type_platforms=type_platforms)
        plotly_reporter.draw_test_case_by_area(filename=filename, cases=random_expected_image["data"])
        assert compare_image(actual=filename, expected=random_expected_image["filename"])
    finally:
        if path.exists(filename):
            remove(filename)


def test_draw_test_case_by_area_changes_ar_colors(
    random_expected_image, compare_image, random_rgb  # pylint: disable=W0621
):
    """
    Init PlotlyReporter and call draw_test_case_by_area with valid parameters and ar_colors should create correct image

    :param random_expected_image: fixture, returns any of possible expected cases
    :param compare_image: fixture, returns function to compare images
    :param random_rgb: fixture, returns random rgb in string format
    """
    type_platforms = [{"name": "UI", "sections": [8080, 11, 4]}, {"name": "API", "sections": [12, 101, 86]}]
    filename = "actual_test_case_by_area_ar_color.png"
    ar_colors = [random_rgb(), random_rgb(), random_rgb(), random_rgb(), random_rgb()]
    try:
        plotly_reporter = PlotlyReporter(type_platforms=type_platforms)
        plotly_reporter.draw_test_case_by_area(
            filename=filename, cases=random_expected_image["data"], ar_colors=ar_colors
        )
        assert not compare_image(actual=filename, expected=random_expected_image["filename"])
    finally:
        if path.exists(filename):
            remove(filename)


def test_draw_test_case_by_area_changes_lines(
    random_expected_image, compare_image, random_rgb  # pylint: disable=W0621
):
    """
    Init PlotlyReporter and call draw_test_case_by_area with valid parameters and ar_colors should create correct image

    :param random_expected_image: fixture, returns any of possible expected cases
    :param compare_image: fixture, returns function to compare images
    :param random_rgb: fixture, returns random rgb in string format
    """
    type_platforms = [{"name": "UI", "sections": [8080, 11, 4]}, {"name": "API", "sections": [12, 101, 86]}]
    filename = "actual_test_case_by_area_lines.png"
    lines = {"color": random_rgb(), "width": float(randint(6, 50)) / 10.0}
    try:
        plotly_reporter = PlotlyReporter(type_platforms=type_platforms)
        plotly_reporter.draw_test_case_by_area(filename=filename, cases=random_expected_image["data"], lines=lines)
        assert not compare_image(actual=filename, expected=random_expected_image["filename"])
    finally:
        if path.exists(filename):
            remove(filename)



# -*- coding: utf-8 -*-
"""Tests for the csv_parser module, 'load_history' function"""

from datetime import datetime

import pytest
from faker import Faker

from testrail_api_reporter.utils.csv_parser import CSVParser  # pylint: disable=import-error,no-name-in-module


fake = Faker()


def test_load_history_data(csv_file, random_stat):
    """Check load_history_data function"""
    parser = CSVParser(filename=csv_file)

    total, automated, not_automated, not_applicable = random_stat
    year = fake.year()
    month = fake.month()
    day_of_month = fake.day_of_month()
    with open(csv_file, "w", encoding="utf-8") as writable_file:
        writable_file.write(f"{year},{month},{day_of_month},{total},{automated},{not_automated},{not_applicable}\n")

    data = parser.load_history_data()

    assert data == [
        [datetime(int(year), int(month), int(day_of_month))],
        [str(total)],
        [str(automated)],
        [str(not_automated)],
        [str(not_applicable)],
    ]


def test_load_history_data_no_filename():
    """No filename is provided for load history"""
    parser = CSVParser()

    with pytest.raises(ValueError):
        parser.load_history_data()


def test_load_history_data_file_not_found():
    """File not found error for load history"""
    parser = CSVParser(filename="non_existent_file.csv")

    with pytest.raises(ValueError):
        parser.load_history_data()



# -*- coding: utf-8 -*-
"""Tests for the csv_parser module against custom logger"""

from logging import Logger, DEBUG, ERROR, FATAL, INFO, WARNING
from random import choice

from faker import Faker

from testrail_api_reporter.utils.csv_parser import CSVParser  # pylint: disable=import-error,no-name-in-module


fake = Faker()


def test_custom_csv_logger():
    """Use custom logger"""
    CSVParser(logger=Logger(name=fake.name(), level=choice((DEBUG, INFO, WARNING, ERROR, FATAL))))



# -*- coding: utf-8 -*-
"""Tests for the logger_config module"""

from logging import DEBUG, INFO, WARNING, ERROR, FATAL, FileHandler, StreamHandler
from os import path, remove
from random import choice

from faker import Faker

from testrail_api_reporter.utils.logger_config import (  # pylint: disable=import-error,no-name-in-module
    setup_logger,
    DEFAULT_LOGGING_LEVEL,
)

fake = Faker()


def test_setup_logger_default_level():
    """Init logger with default level"""
    log_file = fake.file_name(extension="log")
    try:
        logger = setup_logger(fake.name(), str(log_file))

        assert logger.level == DEFAULT_LOGGING_LEVEL
        assert logger.level == DEBUG

        assert len(logger.handlers) == 2
        assert isinstance(logger.handlers[0], FileHandler)
        assert isinstance(logger.handlers[1], StreamHandler)

        message = fake.name()
        logger.debug(message)
        with open(log_file, "r", encoding="utf-8") as readable_file:
            assert message in readable_file.read()
    finally:
        if path.exists(log_file):
            remove(log_file)


def test_setup_logger_custom_level():
    """Init logger with any other level"""
    log_file = fake.file_name(extension="log")
    try:
        log_level = choice((INFO, WARNING, ERROR, FATAL))
        logger = setup_logger(fake.name(), str(log_file), level=log_level)

        assert logger.level == log_level
    finally:
        if path.exists(log_file):
            remove(log_file)



# -*- coding: utf-8 -*-
"""Tests for the reporter_utils module, function 'upload_image'"""

from os import getcwd, getenv
from random import choice
from unittest.mock import patch, Mock

from faker import Faker
from pytest import raises as pytest_raises

from testrail_api_reporter.utils.reporter_utils import upload_image  # pylint: disable=import-error,no-name-in-module


test_filename = choice((f"{getcwd()}/tests/assets/test_image.png", f"{getcwd()}/tests/assets/test_image.jpeg"))


def test_upload_image_mock_success():
    """Test success image upload (mock)"""
    with patch("requests.post") as mock_post:
        faker = Faker()
        url = faker.image_url()
        thumb_url = faker.image_url()
        mock_response = Mock()
        mock_response.json.return_value = {"image": {"url": url, "thumb": {"url": thumb_url}}}
        mock_post.return_value = mock_response
        result = upload_image(test_filename, "test_api_token")
        assert result == {"image": url, "thumb": thumb_url}


def test_upload_image_mock_nonexistent_file():
    """Test response for not existent file (mock)"""
    with pytest_raises(FileNotFoundError):
        faker = Faker()
        upload_image(faker.file_path(extension=choice(("png", "jpg", "jpeg"))), faker.password())


def test_upload_image_mock_invalid_token():
    """Test against invalid token (mock)"""
    with patch("requests.post") as mock_post:
        mock_response = Mock()
        mock_response.json.return_value = {"error": "Invalid API token"}
        mock_post.return_value = mock_response
        with pytest_raises(KeyError):
            upload_image(test_filename, Faker().password())


def test_upload_image_mock_api_error():
    """Test against API error (invalid response/request) (mock)"""
    with patch("requests.post") as mock_post:
        mock_response = Mock()
        mock_response.json.return_value = {"error": "Upload error"}
        mock_post.return_value = mock_response
        with pytest_raises(KeyError):
            upload_image(test_filename, Faker().password())


def test_upload_image_live_success():
    """Test success image upload"""
    result = upload_image(test_filename, getenv("FREEIMAGEHOST_API_KEY"))
    assert "image" in result
    assert "thumb" in result


def test_upload_image_live_nonexistent_file():
    """Test response for not existent file"""
    with pytest_raises(FileNotFoundError):
        upload_image(Faker().file_path(extension=choice(("png", "jpg", "jpeg"))), getenv("FREEIMAGEHOST_API_KEY"))


def test_upload_image_live_invalid_token():
    """Test against invalid token"""
    with pytest_raises(KeyError):
        upload_image(test_filename, Faker().password())



# -*- coding: utf-8 -*-
""" CSV parser for TestRail API Reporter """

import csv
from datetime import datetime
from os.path import exists
from typing import List

from .logger_config import setup_logger, DEFAULT_LOGGING_LEVEL


class CSVParser:
    """Parser for CSV files"""

    def __init__(self, filename=None, logger=None, log_level=DEFAULT_LOGGING_LEVEL):
        """
        Default init

        :param filename: filename for csv file
        :param logger: logger object, optional
        :param log_level: logging level, optional, by default is logging.DEBUG
        """
        if not logger:
            self.___logger = setup_logger(name="CSVParser", log_file="CSVParser.log", level=log_level)
        else:
            self.___logger = logger
        self.___logger.debug("Initializing CSV Parser")
        self.__filename = filename

    def save_history_data(self, filename=None, report=None):
        """
        Save history data to CSV

        :param filename: file name of output file, required
        :param report: report with distribution in CaseStat format
        :return:
        """
        filename = filename if filename else self.__filename
        if not filename:
            raise ValueError("Filename for save report data is not provided, save history data aborted!")
        if not report:
            raise ValueError("Report couldn't be found, save history data aborted!")
        date = datetime.today().strftime("%Y-%m-%d")
        last_date = ""
        mode = "r" if exists(filename) else "w"
        with open(filename, mode, encoding="utf-8") as csvfile:
            if mode == "r":
                for row in reversed(list(csv.reader(csvfile))):
                    last_date = f"{row[0]}-{row[1]}-{row[2]}"
                    break
        if last_date != date:
            self.___logger.debug("Last date in file: %s for %s", filename, last_date)
            with open(filename, "a+", newline="", encoding="utf-8") as csvfile:
                writer = csv.writer(csvfile, delimiter=",", quotechar="|", quoting=csv.QUOTE_MINIMAL)

                writer.writerow(
                    [
                        datetime.today().strftime("%Y"),
                        datetime.today().strftime("%m"),
                        datetime.today().strftime("%d"),
                        report.get_total(),
                        report.get_automated(),
                        report.get_not_automated(),
                        report.get_not_applicable(),
                    ]
                )
        else:
            self.___logger.debug("Data already stored for today, skipping save")

    def load_history_data(self, filename=None) -> List:
        """
        Load history data to CSV

        :param filename: file name of output file, required
        :return: list with results
        """
        filename = filename if filename else self.__filename
        if not filename:
            raise ValueError("Filename for load report data is not provided, save history data aborted!")
        timestamps = []
        totals = []
        automated = []
        not_automated = []
        nas = []
        self.___logger.debug("Loading history data from %s", filename)
        try:
            with open(filename, "r", encoding="utf-8") as csvfile:
                for row in csv.reader(csvfile):
                    timestamps.append(datetime(year=int(row[0]), month=int(row[1]), day=int(row[2])))
                    totals.append(row[3])
                    automated.append(row[4])
                    not_automated.append(row[5])
                    nas.append(row[6])
        except FileNotFoundError:
            raise ValueError(f"Can't open report file '{filename}', load history data aborted!") from FileNotFoundError
        return [timestamps, totals, automated, not_automated, nas]



# -*- coding: utf-8 -*-
"""Logger configuration"""

import logging
import sys

DEFAULT_LOGGING_LEVEL = logging.DEBUG


def setup_logger(name: str, log_file: str, level=logging.DEBUG):
    """
    Method to setup logger

    :param name: (string) Name of the logger.
    :param log_file: path to log_file
    :param level: logging level. Default is logging.DEBUG
    :returns: logger object
    """
    formatter = logging.Formatter("%(asctime)s [%(levelname)s] %(name)s: %(message)s")

    file_handler = logging.FileHandler(log_file)
    file_handler.setFormatter(formatter)
    stream_handler = logging.StreamHandler(sys.stdout)
    stream_handler.setFormatter(formatter)

    logger = logging.getLogger(name)
    logger.setLevel(level)

    logger.addHandler(file_handler)
    logger.addHandler(stream_handler)

    return logger



# -*- coding: utf-8 -*-
"""Tests for the reporter_utils module, function 'delete_file'"""

from os import path, remove
from unittest.mock import MagicMock

from faker import Faker

from testrail_api_reporter.utils.reporter_utils import delete_file  # pylint: disable=import-error,no-name-in-module


def test_delete_file(create_test_file):
    """
    Test delete file

        :param create_test_file: fixture, returns filename of test file to be deleted
    """

    delete_file(create_test_file, debug=False)

    assert path.exists(create_test_file) is False


def test_delete_file_with_debug(create_test_file):
    """
    Test delete file with debug output

    :param create_test_file: fixture, returns filename of test file to be deleted
    """
    mock_logger = MagicMock()
    try:
        delete_file(create_test_file, debug=True, logger=mock_logger)

        assert path.exists(create_test_file) is False
        mock_logger.debug.assert_called_once_with(f"Removed {create_test_file}")
    finally:
        if path.exists(create_test_file):
            remove(create_test_file)


def test_delete_file_non_existent(capfd):
    """
    Test delete non-existing file

    :param capfd - fixture of cap failure logger
    """
    delete_file(f"ne_delete_{Faker().file_name()}", debug=True)
    _, err = capfd.readouterr()

    assert "No such file or directory" in err



# -*- coding: utf-8 -*-
"""Conftest for testsuite"""

from os import path, remove
from random import randint

import pytest
from faker import Faker
from PIL import Image, ImageChops

from testrail_api_reporter.engines.plotly_reporter import (  # pylint: disable=import-error,no-name-in-module
    PlotlyReporter,
)
from testrail_api_reporter.utils.case_stat import CaseStat  # pylint: disable=import-error,no-name-in-module


fake = Faker()


@pytest.fixture
def create_test_file():
    """
    Fixture to create random test file

    :return: filename
    :rtype: str (generator)
    """
    test_file = f"not_existing_{fake.file_name()}"
    with open(test_file, "w", encoding="utf-8") as file:
        file.write("Test")
    assert path.exists(test_file) is True
    yield test_file
    # Cleanup if not removed by tests
    try:
        remove(test_file)
    except FileNotFoundError:
        pass


@pytest.fixture
def random_stat() -> tuple:
    """
    Fixture to return tuple with random statistics

    :return: tuple with random statistics
    :rtype: tuple[int, int, int, int]
    """
    total = randint(0, 32768)
    automated = randint(0, 32768)
    not_automated = randint(0, 32768)
    not_applicable = randint(0, 32768)
    return total, automated, not_automated, not_applicable


@pytest.fixture
def case_stat() -> CaseStat:
    """
    Fixture to return object of CaseStat

    :return: CaseStat
    :rtype: CaseStat
    """
    return CaseStat(fake.word())


@pytest.fixture
def case_stat_random(case_stat, random_stat):  # pylint: disable=redefined-outer-name
    """
    Fixture to return object of CaseStat

    :return: CaseStat with random statistics
    :rtype: CaseStat
    """
    total, automated, not_automated, not_applicable = random_stat
    case_stat.set_total(total)
    case_stat.set_automated(automated)
    case_stat.set_not_automated(not_automated)
    case_stat.set_not_applicable(not_applicable)
    return case_stat


@pytest.fixture
def csv_file():
    """
    Fixture to create random test file

    :return: filename
    :rtype: str (generator)
    """
    test_file = f"not_existing_{fake.file_name(extension='csv')}"
    with open(test_file, "w", encoding="utf-8") as file:
        file.write("")
    assert path.exists(test_file) is True
    yield test_file
    # Cleanup if not removed by tests
    try:
        remove(test_file)
    except FileNotFoundError:
        pass


@pytest.fixture
def compare_image():
    """
    Fixture to compare images using pixel threshold

    :return: comparison function
    :rtype: function
    """

    def compare(actual: str, expected: str, threshold: int = 10) -> bool:
        """
        Function to compare images using pixel threshold

        :param actual: filename with path to actual image
        :type actual: str
        :param expected: filename with path to expected image
        :type actual: str
        :param threshold: pixel difference tolerance between images - lesser is better
        :type actual: int
        :return: comparison result. True if images match.
        :rtype: bool
        """
        # Ensure that images exists
        assert path.exists(actual)
        assert path.exists(expected)

        # Load the generated image and the reference image
        generated_image = Image.open(actual)
        reference_image = Image.open(expected)

        # Compare the two images
        diff = ImageChops.difference(generated_image, reference_image)

        # Count the number of pixels that are different
        diff_pixels = sum(abs(r - g) + abs(g - b) + abs(b - a) + abs(a - r) > 20 for r, g, b, a in diff.getdata())

        # Check that the number of different pixels is below the threshold
        return diff_pixels < threshold

    return compare


@pytest.fixture
def random_type_platforms() -> list:
    """
    Returns random list with type platforms dict

    :return: list with type platforms dict
    :rtype: list[dict]
    """
    return [{"name": fake.word(), "sections": [randint(1, 10000)]} for _ in range(randint(1, 5))]


@pytest.fixture
def random_plotly_reporter(random_type_platforms) -> PlotlyReporter:  # pylint: disable=redefined-outer-name
    """
    Returns PlotlyReporter object with random type platforms

    :return: PlotlyReporter object with random type platforms
    :rtype: PlotlyReporter
    """
    return PlotlyReporter(type_platforms=random_type_platforms)


@pytest.fixture()
def random_rgb():
    """Returns fixture to get rgb in string format"""

    def get_rgb() -> str:
        """
        Returns rgb in string format

        :return: rgb in string format
        :rtype: str
        """
        return f"rgb({randint(0, 255)},{randint(0, 255)},{randint(0, 255)})"

    return get_rgb



# -*- coding: utf-8 -*-
"""Tests for plotly_reporter module, the PlotlyReporter clas, draw_history_state_chart method"""

import pytest
from faker import Faker


fake = Faker()


def test_draw_history_state_chart_no_chart_name(random_plotly_reporter):
    """
    Init PlotlyReporter and call draw_history_state_chart without chart_name should raise ValueError

    :param random_plotly_reporter: fixture returns PlotlyReporter
    """
    with pytest.raises(ValueError, match="No chart name is provided, report aborted!"):
        random_plotly_reporter.draw_history_state_chart()  # type: ignore


def test_draw_history_state_chart_no_filename(random_plotly_reporter):
    """
    Init PlotlyReporter and call draw_test_case_by_priority without filename should raise ValueError

    :param random_plotly_reporter: fixture returns PlotlyReporter
    """
    fake_name = fake.name()
    match = f"Can't open report file 'current_automation_{fake_name.replace(' ', '_')}.csv', load history data aborted!"
    with pytest.raises(ValueError, match=match):
        random_plotly_reporter.draw_history_state_chart(chart_name=fake_name)


def test_draw_history_state_chart_creates_file():
    """Init PlotlyReporter and call draw_history_state_chart with valid parameters should create file"""
    return  # not implemented yet


def test_draw_history_state_chart_creates_correct_image():
    """Init PlotlyReporter and call draw_history_state_chart with valid parameters should create valid image"""
    return  # not implemented yet


def test_draw_history_state_chart_default_history_data():
    """Init PlotlyReporter and call draw_history_state_chart with default history data"""
    return  # not implemented yet


def test_draw_history_state_chart_custom_history_data():
    """Init PlotlyReporter and call draw_history_state_chart with custom history data"""
    return  # not implemented yet


def test_draw_history_state_chart_trace1_decor():
    """Init PlotlyReporter and call draw_history_state_chart with custom trace1 decor"""
    return  # not implemented yet


def test_draw_history_state_chart_trace2_decor():
    """Init PlotlyReporter and call draw_history_state_chart with custom trace2 decor"""
    return  # not implemented yet


def test_draw_history_state_chart_reverse_traces():
    """Init PlotlyReporter and call draw_history_state_chart with reversed traces"""
    return  # not implemented yet


def test_draw_history_state_chart_filename_pattern():
    """Init PlotlyReporter and call draw_history_state_chart with filename pattern"""
    return  # not implemented yet



# -*- coding: utf-8 -*-
"""Tests for plotly_reporter module, the PlotlyReporter clas, draw_test_case_by_priority method"""

from os import path, remove, getcwd, environ
from random import choice, randint

import pytest
from faker import Faker


fake = Faker()


def test_draw_test_case_by_priority_no_cases(random_plotly_reporter):
    """
    Init PlotlyReporter and call draw_test_case_by_priority without cases should raise ValueError

    :param random_plotly_reporter: fixture returns PlotlyReporter
    """
    with pytest.raises(ValueError, match="No TestRail values are provided, report aborted!"):
        random_plotly_reporter.draw_test_case_by_priority(
            filename=fake.file_name(extension=choice(("png", "jpg", "jpeg", "webp")))
        )


def test_draw_test_case_by_priority_no_filename(random_plotly_reporter):
    """
    Init PlotlyReporter and call draw_test_case_by_priority without filename should raise ValueError

    :param random_plotly_reporter: fixture returns PlotlyReporter
    """
    with pytest.raises(ValueError, match="No output filename is provided, report aborted!"):
        random_plotly_reporter.draw_test_case_by_priority(values=[fake.pydict()])


def test_draw_test_case_by_priority_creates_file(random_plotly_reporter):
    """
    Init PlotlyReporter and call draw_test_case_by_priority with valid parameters should create file

    :param random_plotly_reporter: fixture returns PlotlyReporter
    """
    filename = fake.file_name(extension=choice(("png", "jpg", "jpeg", "webp")))
    try:
        random_plotly_reporter.draw_test_case_by_priority(
            filename=filename, values=[randint(0, 1024) for _ in range(randint(1, 4))]
        )

        assert path.exists(filename)
    finally:
        if path.exists(filename):
            remove(filename)


@pytest.mark.xfail(condition="GITHUB_ACTIONS" in environ, reason="Image may differ on GA env")
def test_draw_test_case_by_priority_creates_correct_image(compare_image, random_plotly_reporter):
    """
    Init PlotlyReporter and call draw_test_case_by_priority with valid parameters should create correct image

    :param compare_image: fixture, returns function to compare images
    :param random_plotly_reporter: fixture returns PlotlyReporter
    """
    filename = "actual_test_case_by_priority.png"
    try:
        random_plotly_reporter.draw_test_case_by_priority(filename=filename, values=[1, 101, 72, 16])
        assert compare_image(actual=filename, expected=f"{getcwd()}/tests/assets/expected_case_by_priority.png")
    finally:
        if path.exists(filename):
            remove(filename)


def test_draw_test_case_by_priority_creates_change_lines(compare_image, random_plotly_reporter, random_rgb):
    """
    Init PlotlyReporter and call draw_test_case_by_priority with custom lines

    :param compare_image: fixture, returns function to compare images
    :param random_plotly_reporter: fixture returns PlotlyReporter
    :param random_rgb: fixture, returns random rgb in string format
    """
    filename = "actual_test_case_by_priority_lines.png"
    try:
        lines = {"color": random_rgb(), "width": float(randint(6, 50)) / 10.0}
        random_plotly_reporter.draw_test_case_by_priority(filename=filename, values=[1, 101, 72, 16], lines=lines)
        assert not compare_image(actual=filename, expected=f"{getcwd()}/tests/assets/expected_case_by_priority.png")
    finally:
        if path.exists(filename):
            remove(filename)


def test_draw_test_case_by_priority_creates_change_labels(compare_image, random_plotly_reporter):
    """
    Init PlotlyReporter and call draw_test_case_by_priority with custom labels

    :param compare_image: fixture, returns function to compare images
    :param random_plotly_reporter: fixture returns PlotlyReporter
    """
    filename = "actual_test_case_by_priority_labels.png"
    labels = [fake.name() for _ in range(randint(1, 10))]
    try:
        random_plotly_reporter.draw_test_case_by_priority(filename=filename, values=[1, 101, 72, 16], pr_labels=labels)
        assert not compare_image(actual=filename, expected=f"{getcwd()}/tests/assets/expected_case_by_priority.png")
    finally:
        if path.exists(filename):
            remove(filename)


def test_draw_test_case_by_priority_creates_change_colors(compare_image, random_plotly_reporter, random_rgb):
    """
    Init PlotlyReporter and call draw_test_case_by_priority with custom labels

    :param compare_image: fixture, returns function to compare images
    :param random_plotly_reporter: fixture returns PlotlyReporter
    """
    filename = "actual_test_case_by_priority_colors.png"
    colors = [random_rgb() for _ in range(randint(1, 10))]
    try:
        random_plotly_reporter.draw_test_case_by_priority(filename=filename, values=[1, 101, 72, 16], pr_colors=colors)
        assert not compare_image(actual=filename, expected=f"{getcwd()}/tests/assets/expected_case_by_priority.png")
    finally:
        if path.exists(filename):
            remove(filename)



# -*- coding: utf-8 -*-
"""Tests for plotly_reporter module, the PlotlyReporter clas, draw_automation_state_report method"""

from os import path, remove, getcwd, environ
from random import choice, randint

import pytest
from faker import Faker

from testrail_api_reporter.engines.plotly_reporter import (  # pylint: disable=import-error,no-name-in-module
    PlotlyReporter,
)


fake = Faker()


@pytest.fixture
def random_expected_image(case_stat):
    """
    Fixture that chooses random expected image for draw automation state

    :param case_stat: fixture returns empty CaseStat object
    """
    if choice((False, True)):
        case_stat.set_name("Automation State")
        case_stat.total = 5905
        case_stat.automated = 19100
        case_stat.not_automated = 27205
        case_stat.not_applicable = 10092
        return {"filename": f"{getcwd()}/tests/assets/expected_automation_state.png", "data": [case_stat]}
    case_stat.set_name("Automation State")
    return {"filename": f"{getcwd()}/tests/assets/expected_automation_state_empty.png", "data": [case_stat]}


def test_draw_automation_state_report_no_reports(random_plotly_reporter):
    """
    Init PlotlyReporter and call draw_automation_state_report without reports should raise ValueError

    :param random_plotly_reporter: fixture returns PlotlyReporter
    """
    with pytest.raises(ValueError, match="No TestRail reports are provided, report aborted!"):
        random_plotly_reporter.draw_automation_state_report(
            filename=fake.file_name(extension=choice(("png", "jpg", "jpeg", "webp")))
        )


def test_draw_automation_state_report_no_filename(random_plotly_reporter):
    """
    Init PlotlyReporter and call draw_automation_state_report without filename should raise ValueError

    :param random_plotly_reporter: fixture returns PlotlyReporter
    """
    with pytest.raises(ValueError, match="No output filename is provided, report aborted!"):
        random_plotly_reporter.draw_automation_state_report(reports=[fake.pydict()])


def test_draw_automation_state_report_creates_file(case_stat, case_stat_random, random_plotly_reporter):
    """
    Init PlotlyReporter and call draw_automation_state_report with valid parameters should create file

    :param case_stat: fixture returns empty CaseStat object
    :param case_stat_random: fixture returns filled with random data CaseStat object
    :param random_plotly_reporter: fixture returns PlotlyReporter
    """
    filename = fake.file_name(extension=choice(("png", "jpg", "jpeg", "webp")))
    try:
        reports = [case_stat, case_stat_random]
        random_plotly_reporter.draw_automation_state_report(filename=filename, reports=reports)

        assert path.exists(filename)
    finally:
        if path.exists(filename):
            remove(filename)


@pytest.mark.xfail(condition="GITHUB_ACTIONS" in environ, reason="Image may differ on GA env")
def test_draw_automation_state_report_creates_correct_image(
    random_expected_image, compare_image  # pylint: disable=W0621
):
    """
    Init PlotlyReporter and call draw_automation_state_report with valid parameters should create correct image

    :param random_expected_image: fixture, returns any of possible expected cases
    :param compare_image: fixture, returns function to compare images
    """
    type_platforms = [{"name": "Automation State", "sections": [42, 1024, 0]}]
    filename = "actual_automation_state.png"
    try:
        plotly_reporter = PlotlyReporter(type_platforms=type_platforms)
        plotly_reporter.draw_automation_state_report(filename=filename, reports=random_expected_image["data"])
        assert compare_image(actual=filename, expected=random_expected_image["filename"])
    finally:
        if path.exists(filename):
            remove(filename)


def test_draw_automation_state_report_changes_state_markers(
    random_expected_image, compare_image, random_rgb  # pylint: disable=W0621
):
    """
    Init PlotlyReporter and call draw_automation_state_report with state_markers should create different image

    :param random_expected_image: fixture, returns any of possible expected cases
    :param compare_image: fixture, returns function to compare images
    :param random_rgb: fixture, returns random rgb in string format
    """
    type_platforms = [{"name": "Automation State", "sections": [42, 1024, 0]}]
    filename = "actual_automation_state_with_markers.png"
    state_markers = {
        "Automated": {
            "marker": {"color": random_rgb(), "line": {"color": random_rgb(), "width": float(randint(5, 20)) / 10.0}},
            "opacity": float(randint(0, 10)) / 10,
            "textposition": "auto",
        },
        "Not automated": {
            "marker": {"color": random_rgb(), "line": {"color": random_rgb(), "width": float(randint(5, 20)) / 10.0}},
            "opacity": float(randint(0, 10)) / 10,
            "textposition": "auto",
        },
        "N/A": {
            "marker": {"color": random_rgb(), "line": {"color": random_rgb(), "width": float(randint(5, 20)) / 10}},
            "opacity": float(randint(0, 10)) / 10,
            "textposition": "auto",
        },
    }
    try:
        plotly_reporter = PlotlyReporter(type_platforms=type_platforms)
        plotly_reporter.draw_automation_state_report(
            filename=filename, reports=random_expected_image["data"], state_markers=state_markers
        )
        assert not compare_image(actual=filename, expected=random_expected_image["filename"], threshold=1000)
    finally:
        if path.exists(filename):
            remove(filename)



# -*- coding: utf-8 -*-
""" Confluence sender module """

from atlassian import Confluence

from ..engines.plotly_reporter import PlotlyReporter
from ..utils.logger_config import setup_logger, DEFAULT_LOGGING_LEVEL


class ConfluenceSender:
    """Class contains wrapper for generating and sends reports to Confluence"""

    def __init__(
        self,
        url=None,
        username=None,
        password=None,
        confluence_page=None,
        automation_platforms=None,
        type_platforms=None,
        plotly_engine=None,
        logger=None,
        log_level=DEFAULT_LOGGING_LEVEL,
    ):
        """
        General init

        :param url: url of Confluence, string, required
        :param username: username of Confluence user with proper access rights, string, required
        :param password: password of Confluence user with proper access rights, string, required
        :param confluence_page: confluence page short URL, string - only last part of it (it's id or str), optional
        :param automation_platforms: list of dicts of automation platforms, dict = {'name': 'Desktop Chrome',
                                                                                    'internal_name': 'type_id',
                                                                                    'sections': [16276]}, optional
        :param type_platforms: list of dicts, with sections ids, where dict = {'name': 'UI',
                                                                               'sections': [16276]}, optional
        :param plotly_engine: custom graphic reporter engine (PlotlyReporter), if none is selected, new will be created
        :param logger: logger object, optional
        :param log_level: logging level, optional, by default is 'logging.DEBUG'
        """
        if not logger:
            self.___logger = setup_logger(name="ConfluenceReporter", log_file="ConfluenceReporter.log", level=log_level)
        else:
            self.___logger = logger
        self.___logger.debug("ConfluenceReporter init")
        if url is None or username is None or password is None:
            raise ValueError("No confluence credentials are provided!")
        self.__confluence = Confluence(url=url, username=username, password=password)
        self.__confluence_page = confluence_page  # confluence page may vary for each report if needed, None is possible
        self.__plotly = (
            plotly_engine if plotly_engine else PlotlyReporter(type_platforms=type_platforms, log_level=log_level)
        )
        self.__automation_platforms = automation_platforms  # should be passed with specific TestRails sections
        self.__type_platforms = type_platforms

    def automation_state(self, confluence_page=None, reports=None, filename="current_automation.png"):
        """
        Generates and sends (attach) an image file (png) to confluence page with staked distribution (bar chart)
        with automation type coverage (or similar).

        :param confluence_page: confluence page short URL, string - only last part of it (it's id or str), optional
        :param reports: report with stacked distribution, usually it output of
                        ATCoverageReporter().automation_state_report()
        :param filename: filename of image (with a valid path), png expected
        :return: none
        """
        if not confluence_page:
            raise ValueError("No confluence page is provided, report aborted!")
        if not reports:
            raise ValueError("No TestRail reports are provided, report aborted!")
        self.__plotly.draw_automation_state_report(reports=reports, filename=filename)
        self.__confluence.attach_file(filename, page_id=confluence_page, title="current_automation")

    def test_case_priority_distribution(
        self, confluence_page=None, values=None, filename="current_priority_distribution.png"
    ):
        """
        Generates and sends (attach) an image file (png) to confluence page with priority distribution (pie chart)

        :param confluence_page: confluence page short URL, string - only last part of it (it's id or str), optional
        :param values: list of values to draw a report with priority distribution, usually it's output from
                       ATCoverageReporter().test_case_by_priority()
        :param filename: filename of image (maybe with a valid path), png expected
        :return: none
        """
        if not confluence_page:
            raise ValueError("No confluence page is provided, report aborted!")
        if not values:
            raise ValueError("No TestRail reports are provided, report aborted!")
        self.__plotly.draw_test_case_by_priority(values=values, filename=filename)
        self.__confluence.attach_file(filename, page_id=confluence_page, title="current_priority_distribution")

    def test_case_area_distribution(self, confluence_page=None, cases=None, filename="current_area_distribution.png"):
        """
        Generates and sends (attach) an image file (png) to confluence page with sections distribution (pie chart)

        :param confluence_page: confluence page short URL, string - only last part of it (it's id or str), optional
        :param cases: list of values to draw a report with priority distribution, usually it's output from
                       ATCoverageReporter().test_case_by_type()
        :param filename: filename of image (maybe with a valid path), png expected
        :return: none
        """
        if not confluence_page:
            raise ValueError("No confluence page is provided, report aborted!")
        if not cases:
            raise ValueError("No TestRail cases are provided, report aborted!")
        self.__plotly.draw_test_case_by_area(cases=cases, filename=filename)
        self.__confluence.attach_file(filename, page_id=confluence_page, title="current_area_distribution")

    def history_state_chart(self, confluence_page=None, automation_platforms=None):
        """
        Generates and sends (attach) an image file (png) to confluence page with state distribution (staked line chart)

        :param confluence_page: confluence page short URL, string - only last part of it (it's id or str), optional
        :param automation_platforms: list of dicts contains automation platforms = [{'name': 'Desktop Chrome',
                                                                                     'internal_name': 'type_id',
                                                                                     'sections': [16276]}]
        :return: none
        """
        automation_platforms = automation_platforms if automation_platforms else self.__automation_platforms
        if not confluence_page:
            raise ValueError("No confluence page is provided, report aborted!")
        if automation_platforms is None:
            raise ValueError("No automation platforms provided, report aborted!")
        for item in automation_platforms:
            self.___logger.debug("generating chart for %s", item["name"])
            filename = self.__plotly.draw_history_state_chart(chart_name=item["name"])
            self.__confluence.attach_file(filename, page_id=confluence_page, title=filename[:-4])

    def history_type_chart(
        self, confluence_page=None, type_platforms=None, filename="current_area_distribution_history.png"
    ):
        """
        Generates and sends (attach) an image file (png) to confluence page with state distribution (staked line chart)

        :param confluence_page: confluence page short URL, string - only last part of it (it's id or str), optional
        :param type_platforms: list of dicts, with sections ids, where dict = {'name': 'UI',
                                                                               'sections': [16276]}
        :param filename: filename of image (maybe with valid path), png expected
        :return: none
        """
        type_platforms = type_platforms if type_platforms else self.__type_platforms
        if not confluence_page:
            raise ValueError("No confluence page is provided, report aborted!")
        if not type_platforms:
            raise ValueError("No type platforms specified, report aborted!")
        self.__plotly.draw_history_type_chart(type_platforms=type_platforms, filename=filename)
        self.__confluence.attach_file(filename, page_id=confluence_page, title="current_area_distribution_history")

    def generate_report(
        self,
        confluence_page=None,
        reports=None,
        cases=None,
        values=None,
        type_platforms=None,
        automation_platforms=None,
    ):
        """
        Generates and sends (attach) an image file (png) to confluence page with state distribution (staked line chart)

        :param confluence_page: confluence page short URL, string - only last part of it (it's id or str), optional
        :param reports: report with stacked distribution, usually it's output of
                        ATCoverageReporter().automation_state_report()
        :param cases: list of values to draw report with priority distribution, usually it's output from
                      ATCoverageReporter().test_case_by_type()
        :param values: list of values to draw report with priority distribution, usually it's output from
                       ATCoverageReporter().test_case_by_priority()
        :param type_platforms: list of dicts, with sections ids, where dict = {'name': 'UI',
                                                                               'sections': [16276]}, optional
        :param automation_platforms: list of dicts of automation platforms, dict = {'name': 'Desktop Chrome',
                                                                                    'internal_name': 'type_id',
                                                                                    'sections': [16276]}, optional
        :return: none
        """
        confluence_page = confluence_page if confluence_page else self.__confluence_page
        if not confluence_page:
            raise ValueError("No confluence page is provided, report aborted!")
        if not reports:
            raise ValueError("No TestRail reports are provided, report aborted!")
        if not cases:
            raise ValueError("No TestRail cases are provided, report aborted!")
        if not values:
            raise ValueError("No TestRail values are provided, report aborted!")
        type_platforms = type_platforms if type_platforms else self.__type_platforms
        if not type_platforms:
            raise ValueError("No type platforms specified, report aborted!")
        automation_platforms = automation_platforms if automation_platforms else self.__automation_platforms
        if not automation_platforms:
            raise ValueError("No type platforms specified, report aborted!")
        self.automation_state(confluence_page=confluence_page, reports=reports)
        self.test_case_priority_distribution(confluence_page=confluence_page, values=values)
        self.test_case_area_distribution(confluence_page=confluence_page, cases=cases)
        self.history_type_chart(confluence_page=confluence_page)
        self.history_state_chart(confluence_page=confluence_page)



# -*- coding: utf-8 -*-
"""Tests for plotly_reporter module, the PlotlyReporter class, init method"""

from logging import getLogger, INFO, WARNING, ERROR, FATAL
from os import path, remove
from random import randint, choice

import pytest
from faker import Faker

from testrail_api_reporter.engines.plotly_reporter import (  # pylint: disable=import-error,no-name-in-module
    PlotlyReporter,
)
from testrail_api_reporter.utils.logger_config import (  # pylint: disable=import-error,no-name-in-module
    setup_logger,
    DEFAULT_LOGGING_LEVEL,
)

fake = Faker()


def test_plotly_reporter_init_default_params():
    """Init PlotlyReporter with default parameters"""
    type_platforms = [{"name": fake.word(), "sections": [randint(1, 10000)]} for _ in range(randint(1, 5))]

    plotly_reporter = PlotlyReporter(type_platforms=type_platforms)

    logger = getLogger("PlotlyReporter")
    assert logger.level == DEFAULT_LOGGING_LEVEL
    assert path.exists("PlotlyReporter.log")

    attributes = vars(plotly_reporter)
    assert attributes["_PlotlyReporter__pr_labels"] == ["Low", "Medium", "High", "Critical"]
    assert attributes["_PlotlyReporter__pr_colors"] == [
        "rgb(173,216,230)",
        "rgb(34,139,34)",
        "rgb(255,255,51)",
        "rgb(255, 153, 153)",
    ]
    assert attributes["_PlotlyReporter__ar_colors"] == [
        "rgb(255, 153, 153)",
        "rgb(255,255,51)",
        "rgb(34,139,34)",
        "rgb(173,216,230)",
        "rgb(65,105,225)",
        "rgb(192, 192, 192)",
    ]
    assert attributes["_PlotlyReporter__lines"] == {"color": "rgb(0,0,51)", "width": 1.5}
    assert attributes["_PlotlyReporter__type_platforms"] == type_platforms


def test_plotly_reporter_init_custom_params():
    """Init PlotlyReporter with custom parameters"""
    logger_file = fake.file_name(extension="log")
    logger_name = fake.name()
    logger_level = choice((INFO, WARNING, ERROR, FATAL))
    try:
        logger = setup_logger(logger_name, logger_file, level=logger_level)
        type_platforms = [{"name": fake.word(), "sections": [randint(1, 10000)]} for _ in range(randint(1, 5))]
        pr_labels = [fake.word() for _ in range(4)]
        pr_colors = [f"rgb({randint(0, 255)},{randint(0, 255)},{randint(0, 255)})" for _ in range(4)]
        ar_colors = [f"rgb({randint(0, 255)},{randint(0, 255)},{randint(0, 255)})" for _ in range(6)]
        lines = {"color": f"rgb({randint(0, 255)},{randint(0, 255)},{randint(0, 255)})", "width": randint(1, 3)}

        plotly_reporter = PlotlyReporter(
            pr_colors=pr_colors,
            pr_labels=pr_labels,
            ar_colors=ar_colors,
            lines=lines,
            type_platforms=type_platforms,
            logger=logger,
            log_level=INFO,
        )

        logger = getLogger(logger_name)
        assert logger.level == logger_level
        assert path.exists(logger_file)

        attributes = vars(plotly_reporter)
        assert attributes["_PlotlyReporter__pr_labels"] == pr_labels
        assert attributes["_PlotlyReporter__pr_colors"] == pr_colors
        assert attributes["_PlotlyReporter__ar_colors"] == ar_colors
        assert attributes["_PlotlyReporter__lines"] == lines
        assert attributes["_PlotlyReporter__type_platforms"] == type_platforms
    finally:
        if path.exists(logger_file):
            remove(logger_file)


def test_plotly_reporter_init_no_type_platforms():
    """Init PlotlyReporter without type_platforms should raise ValueError"""
    with pytest.raises(ValueError, match="Platform types is not provided, Plotly Reporter cannot be initialized!"):
        PlotlyReporter()
